@@ -1,15 +1,33 @@
-use core::{ptr, slice, time};
-use std::convert::TryInto;
+use core::convert::TryInto;
+use core::ptr;
+use core::slice;
+use core::time;
 
 extern crate alloc;
 use alloc::{borrow::Cow, sync::Arc};
 
-use embedded_svc::mqtt::client;
+use embedded_svc::{mqtt::client, service};
+
+use esp_idf_hal::mutex::{Condvar, Mutex};
+
 use esp_idf_sys::*;
 
-use crate::private::cstr::*;
+use crate::private::{common::Newtype, cstr::*};
 
-// !!! NOTE: WORK IN PROGRESS
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+pub enum MqttProtocolVersion {
+    V3_1,
+    V3_1_1,
+}
+
+impl From<MqttProtocolVersion> for esp_mqtt_protocol_ver_t {
+    fn from(pv: MqttProtocolVersion) -> Self {
+        match pv {
+            MqttProtocolVersion::V3_1 => esp_mqtt_protocol_ver_t_MQTT_PROTOCOL_V_3_1,
+            MqttProtocolVersion::V3_1_1 => esp_mqtt_protocol_ver_t_MQTT_PROTOCOL_V_3_1_1,
+        }
+    }
+}
 
 #[derive(Debug)]
 pub struct LwtConfiguration<'a> {
@@ -19,86 +37,179 @@ pub struct LwtConfiguration<'a> {
     pub retain: bool,
 }
 
-#[derive(Debug, Default)]
-pub struct Configuration<'a> {
-    // pub protocol_version: ProtocolVersion,
+#[derive(Debug)]
+pub struct MqttClientConfiguration<'a> {
+    pub protocol_version: Option<MqttProtocolVersion>,
+
     pub client_id: Option<&'a str>,
 
     pub connection_refresh_interval: time::Duration,
     pub keep_alive_interval: Option<time::Duration>,
-    pub reconnect_timeout: time::Duration,
+    pub reconnect_timeout: Option<time::Duration>,
     pub network_timeout: time::Duration,
 
     pub lwt: Option<LwtConfiguration<'a>>,
 
     pub disable_clean_session: bool,
-    pub disable_auto_reconnect: bool,
 
     pub task_prio: u8,
     pub task_stack: usize,
     pub buffer_size: usize,
     pub out_buffer_size: usize,
+
+    pub use_global_ca_store: bool,
+    pub skip_cert_common_name_check: bool,
+    #[cfg(not(esp_idf_version = "4.3"))]
+    pub crt_bundle_attach: Option<unsafe extern "C" fn(conf: *mut c_void) -> esp_err_t>,
+    // TODO: Future
+
     // pub cert_pem: &'a [u8],
     // pub client_cert_pem: &'a [u8],
     // pub client_key_pem: &'a [u8],
 
     // pub psk_hint_key: KeyHint,
-    // pub use_global_ca_store: bool,
-    // //esp_err_t (*crt_bundle_attach)(void *conf); /*!< Pointer to ESP x509 Certificate Bundle attach function for the usage of certification bundles in mqtts */
     // pub alpn_protos: &'a [&'a str],
 
     // pub clientkey_password: &'a str,
-    // pub skip_cert_common_name_check: bool,
     // pub use_secure_element: bool,
 
     // void *ds_data;                          /*!< carrier of handle for digital signature parameters */
 }
 
-impl<'a> From<&Configuration<'a>> for (esp_mqtt_client_config_t, RawCstrs) {
-    fn from(conf: &Configuration<'a>) -> Self {
+impl<'a> Default for MqttClientConfiguration<'a> {
+    fn default() -> Self {
+        Self {
+            protocol_version: None,
+
+            client_id: None,
+
+            connection_refresh_interval: time::Duration::from_secs(0),
+            keep_alive_interval: Some(time::Duration::from_secs(0)),
+            reconnect_timeout: Some(time::Duration::from_secs(0)),
+            network_timeout: time::Duration::from_secs(0),
+
+            lwt: None,
+
+            disable_clean_session: false,
+
+            task_prio: 0,
+            task_stack: 0,
+            buffer_size: 0,
+            out_buffer_size: 0,
+
+            use_global_ca_store: false,
+            skip_cert_common_name_check: false,
+
+            #[cfg(not(esp_idf_version = "4.3"))]
+            crt_bundle_attach: Default::default(),
+        }
+    }
+}
+
+impl<'a> From<&MqttClientConfiguration<'a>> for (esp_mqtt_client_config_t, RawCstrs) {
+    fn from(conf: &MqttClientConfiguration<'a>) -> Self {
         let mut cstrs = RawCstrs::new();
 
-        let c_conf = esp_mqtt_client_config_t {
+        let mut c_conf = esp_mqtt_client_config_t {
+            protocol_ver: if let Some(protocol_version) = conf.protocol_version {
+                protocol_version.into()
+            } else {
+                esp_mqtt_protocol_ver_t_MQTT_PROTOCOL_UNDEFINED
+            },
             client_id: cstrs.as_nptr(conf.client_id),
-            // refresh_connection: time::Duration,
-            // reconnect_timeout: time::Duration,
-            // network_timeout: time::Duration,
-            // keepalive: Option<time::Duration>,
+
+            refresh_connection_after_ms: conf.connection_refresh_interval.as_millis() as _,
+            network_timeout_ms: conf.network_timeout.as_millis() as _,
+
+            disable_clean_session: conf.disable_clean_session as _,
+
+            task_prio: conf.task_prio as _,
+            task_stack: conf.task_stack as _,
+            buffer_size: conf.buffer_size as _,
+            out_buffer_size: conf.out_buffer_size as _,
+
+            use_global_ca_store: conf.use_global_ca_store,
+            skip_cert_common_name_check: conf.skip_cert_common_name_check,
+            #[cfg(not(esp_idf_version = "4.3"))]
+            crt_bundle_attach: conf.crt_bundle_attach,
+
             ..Default::default()
         };
 
+        if let Some(keep_alive_interval) = conf.keep_alive_interval {
+            c_conf.keepalive = keep_alive_interval.as_secs() as _;
+            c_conf.keepalive = true as _;
+        } else {
+            c_conf.keepalive = false as _;
+        }
+
+        if let Some(reconnect_timeout) = conf.reconnect_timeout {
+            c_conf.reconnect_timeout_ms = reconnect_timeout.as_millis() as _;
+            c_conf.disable_auto_reconnect = false;
+        } else {
+            c_conf.disable_auto_reconnect = true;
+        }
+
+        if let Some(lwt) = conf.lwt.as_ref() {
+            c_conf.lwt_topic = cstrs.as_ptr(lwt.topic);
+            c_conf.lwt_msg = lwt.payload.as_ptr() as _;
+            c_conf.lwt_msg_len = lwt.payload.len() as _;
+            c_conf.lwt_qos = lwt.qos as _;
+            c_conf.lwt_retain = lwt.retain as _;
+        }
+
         (c_conf, cstrs)
     }
 }
 
+struct UnsafeCallback(*mut Box<dyn FnMut(esp_mqtt_event_handle_t)>);
+
+impl UnsafeCallback {
+    fn from(boxed: &mut Box<Box<dyn FnMut(esp_mqtt_event_handle_t)>>) -> Self {
+        Self(boxed.as_mut())
+    }
+
+    unsafe fn from_ptr(ptr: *mut c_types::c_void) -> Self {
+        Self(ptr as *mut _)
+    }
+
+    fn as_ptr(&self) -> *mut c_types::c_void {
+        self.0 as *mut _
+    }
+
+    unsafe fn call(&self, data: esp_mqtt_event_handle_t) {
+        let reference = self.0.as_mut().unwrap();
+
+        (reference)(data);
+    }
+}
+
 pub struct EspMqttClient(
     esp_mqtt_client_handle_t,
-    Box<dyn Fn(esp_mqtt_event_handle_t)>,
+    Box<dyn FnMut(esp_mqtt_event_handle_t)>,
 );
 
 impl EspMqttClient {
     pub fn new<'a>(
         url: impl AsRef<str>,
-        conf: &'a Configuration<'a>,
-    ) -> Result<(Self, EspConnection), EspError>
+        conf: &'a MqttClientConfiguration<'a>,
+    ) -> Result<(Self, EspMqttConnection), EspError>
     where
         Self: Sized,
     {
-        let queue = unsafe {
-            xQueueGenericCreate(1, core::mem::size_of::<&esp_mqtt_event_handle_t>() as _, 0)
-        };
-        if queue.is_null() {
-            esp!(ESP_FAIL)?;
-        }
+        let state = Arc::new(EspMqttConnectionState {
+            message: Mutex::new(None),
+            posted: Condvar::new(),
+            processed: Condvar::new(),
+        });
 
-        let queue = Arc::new(Queue(queue));
-
-        let connection = EspConnection(queue.clone());
+        let connection = EspMqttConnection(state.clone());
+        let client_connection = connection.clone();
 
         let client = Self::new_with_raw_callback(
             url,
             conf,
-            Box::new(move |event_handle| EspConnection::post(&queue, event_handle)),
+            Box::new(move |event_handle| EspMqttConnection::post(&client_connection, event_handle)),
         )?;
 
         Ok((client, connection))
@@ -106,8 +217,9 @@ impl EspMqttClient {
 
     pub fn new_with_callback<'a>(
         url: impl AsRef<str>,
-        conf: &'a Configuration<'a>,
-        callback: impl for<'b> Fn(Result<client::Event<EspMessage<'b>>, EspError>) + 'static,
+        conf: &'a MqttClientConfiguration<'a>,
+        mut callback: impl for<'b> FnMut(Option<Result<client::Event<EspMqttMessage<'b>>, EspError>>)
+            + 'static,
     ) -> Result<Self, EspError>
     where
         Self: Sized,
@@ -116,21 +228,29 @@ impl EspMqttClient {
             url,
             conf,
             Box::new(move |event_handle| {
-                let event = unsafe { event_handle.as_ref().unwrap() };
+                let event = unsafe { event_handle.as_ref() };
 
-                callback(EspMessage::new_event(event, None))
+                if let Some(event) = event {
+                    callback(Some(EspMqttMessage::new_event(event, None)))
+                } else {
+                    callback(None)
+                }
             }),
         )
     }
 
     fn new_with_raw_callback<'a>(
         url: impl AsRef<str>,
-        conf: &'a Configuration<'a>,
-        raw_callback: Box<dyn Fn(esp_mqtt_event_handle_t)>,
+        conf: &'a MqttClientConfiguration<'a>,
+        raw_callback: Box<dyn FnMut(esp_mqtt_event_handle_t)>,
     ) -> Result<Self, EspError>
     where
         Self: Sized,
     {
+        let mut boxed_raw_callback = Box::new(raw_callback);
+
+        let unsafe_callback = UnsafeCallback::from(&mut boxed_raw_callback);
+
         let (c_conf, _cstrs) = conf.into();
 
         let client = unsafe { esp_mqtt_client_init(&c_conf as *const _) };
@@ -138,7 +258,7 @@ impl EspMqttClient {
             esp!(ESP_FAIL)?;
         }
 
-        let client = Self(client, Box::new(raw_callback));
+        let client = Self(client, boxed_raw_callback);
 
         let c_url = CString::new(url.as_ref()).unwrap();
 
@@ -149,7 +269,7 @@ impl EspMqttClient {
                 client.0,
                 esp_mqtt_event_id_t_MQTT_EVENT_ANY,
                 Some(Self::handle),
-                &*client.1 as *const _ as *mut _,
+                unsafe_callback.as_ptr(),
             )
         })?;
 
@@ -164,11 +284,9 @@ impl EspMqttClient {
         _event_id: i32,
         event_data: *mut c_types::c_void,
     ) {
-        let handler_ptr = event_handler_arg as *mut Box<dyn Fn(esp_mqtt_event_handle_t)>;
-
-        let handler = unsafe { handler_ptr.as_ref() }.unwrap();
-
-        (handler)(event_data as _);
+        unsafe {
+            UnsafeCallback::from_ptr(event_handler_arg).call(event_data as _);
+        }
     }
 
     fn check(result: i32) -> Result<client::MessageId, EspError> {
@@ -185,12 +303,40 @@ impl Drop for EspMqttClient {
         esp!(unsafe { esp_mqtt_client_disconnect(self.0) }).unwrap();
         esp!(unsafe { esp_mqtt_client_stop(self.0) }).unwrap();
         esp!(unsafe { esp_mqtt_client_destroy(self.0) }).unwrap();
+
+        (self.1)(ptr::null_mut() as *mut _);
     }
 }
 
-impl client::Client for EspMqttClient {
+impl service::Service for EspMqttClient {
     type Error = EspError;
+}
 
+impl client::Client for EspMqttClient {
+    fn subscribe<'a, S>(
+        &'a mut self,
+        topic: S,
+        qos: client::QoS,
+    ) -> Result<client::MessageId, Self::Error>
+    where
+        S: Into<Cow<'a, str>>,
+    {
+        let c_topic = CString::new(topic.into().as_ref()).unwrap();
+
+        Self::check(unsafe { esp_mqtt_client_subscribe(self.0, c_topic.as_ptr(), qos as _) })
+    }
+
+    fn unsubscribe<'a, S>(&'a mut self, topic: S) -> Result<client::MessageId, Self::Error>
+    where
+        S: Into<Cow<'a, str>>,
+    {
+        let c_topic = CString::new(topic.into().as_ref()).unwrap();
+
+        Self::check(unsafe { esp_mqtt_client_unsubscribe(self.0, c_topic.as_ptr()) })
+    }
+}
+
+impl client::Publish for EspMqttClient {
     fn publish<'a, S, V>(
         &'a mut self,
         topic: S,
@@ -217,44 +363,52 @@ impl client::Client for EspMqttClient {
             )
         })
     }
+}
 
-    fn subscribe<'a, S>(
+impl client::Enqueue for EspMqttClient {
+    fn enqueue<'a, S, V>(
         &'a mut self,
         topic: S,
         qos: client::QoS,
+        retain: bool,
+        payload: V,
     ) -> Result<client::MessageId, Self::Error>
     where
         S: Into<Cow<'a, str>>,
+        V: Into<Cow<'a, [u8]>>,
     {
         let c_topic = CString::new(topic.into().as_ref()).unwrap();
 
-        Self::check(unsafe { esp_mqtt_client_subscribe(self.0, c_topic.as_ptr(), qos as _) })
-    }
+        let payload = payload.into();
 
-    fn unsubscribe<'a, S>(&'a mut self, topic: S) -> Result<client::MessageId, Self::Error>
-    where
-        S: Into<Cow<'a, str>>,
-    {
-        let c_topic = CString::new(topic.into().as_ref()).unwrap();
-
-        Self::check(unsafe { esp_mqtt_client_unsubscribe(self.0, c_topic.as_ptr()) })
+        Self::check(unsafe {
+            esp_mqtt_client_enqueue(
+                self.0,
+                c_topic.as_ptr(),
+                payload.as_ref().as_ptr() as _,
+                payload.as_ref().len() as _,
+                qos as _,
+                retain as _,
+                true,
+            )
+        })
     }
 }
 
 unsafe impl Send for EspMqttClient {}
 
-pub struct EspMessage<'a> {
+pub struct EspMqttMessage<'a> {
     event: &'a esp_mqtt_event_t,
     details: client::Details,
-    queue: Option<Arc<Queue>>,
+    connection: Option<Arc<EspMqttConnectionState>>,
 }
 
-impl<'a> EspMessage<'a> {
+impl<'a> EspMqttMessage<'a> {
     #[allow(non_upper_case_globals)]
     fn new_event(
-        event: &esp_mqtt_event_t,
-        queue: Option<Arc<Queue>>,
-    ) -> Result<client::Event<EspMessage<'_>>, EspError> {
+        event: &'a esp_mqtt_event_t,
+        connection: Option<&Arc<EspMqttConnectionState>>,
+    ) -> Result<client::Event<EspMqttMessage<'a>>, EspError> {
         match event.event_id {
             esp_mqtt_event_id_t_MQTT_EVENT_ERROR => Err(EspError::from(ESP_FAIL).unwrap()), // TODO
             esp_mqtt_event_id_t_MQTT_EVENT_BEFORE_CONNECT => Ok(client::Event::BeforeConnect),
@@ -271,19 +425,19 @@ impl<'a> EspMessage<'a> {
             esp_mqtt_event_id_t_MQTT_EVENT_PUBLISHED => {
                 Ok(client::Event::Published(event.msg_id as _))
             }
-            esp_mqtt_event_id_t_MQTT_EVENT_DATA => {
-                Ok(client::Event::Received(EspMessage::new(event, queue)))
-            }
+            esp_mqtt_event_id_t_MQTT_EVENT_DATA => Ok(client::Event::Received(
+                EspMqttMessage::new(event, connection.map(|connection| connection.clone())),
+            )),
             esp_mqtt_event_id_t_MQTT_EVENT_DELETED => Ok(client::Event::Deleted(event.msg_id as _)),
             other => panic!("Unknown message type: {}", other),
         }
     }
 
-    fn new(event: &'a esp_mqtt_event_t, queue: Option<Arc<Queue>>) -> Self {
+    fn new(event: &'a esp_mqtt_event_t, connection: Option<Arc<EspMqttConnectionState>>) -> Self {
         let mut message = Self {
             event,
             details: client::Details::Complete(unsafe { client::TopicToken::new() }),
-            queue,
+            connection,
         };
 
         message.fill_chunk_details();
@@ -308,24 +462,20 @@ impl<'a> EspMessage<'a> {
     }
 }
 
-impl<'a> Drop for EspMessage<'a> {
+impl<'a> Drop for EspMqttMessage<'a> {
     fn drop(&mut self) {
-        if let Some(queue) = self.queue.as_ref() {
-            let mut conn_data: esp_mqtt_event_handle_t = ptr::null_mut();
+        if let Some(state) = self.connection.as_ref() {
+            let mut message = state.message.lock();
 
-            esp!(unsafe {
-                xQueueReceive(
-                    queue.0,
-                    &mut conn_data as *mut _ as *mut _,
-                    TickType_t::max_value(),
-                )
-            })
-            .unwrap();
+            if message.is_some() {
+                *message = None;
+                state.processed.notify_all();
+            }
         }
     }
 }
 
-impl<'a> client::Message for EspMessage<'a> {
+impl<'a> client::Message for EspMqttMessage<'a> {
     fn id(&self) -> client::MessageId {
         self.event.msg_id as _
     }
@@ -345,7 +495,7 @@ impl<'a> client::Message for EspMessage<'a> {
 
         unsafe {
             let slice = slice::from_raw_parts(ptr as _, len.try_into().unwrap());
-            Cow::Borrowed(std::str::from_utf8(slice).unwrap())
+            Cow::Borrowed(core::str::from_utf8(slice).unwrap())
         }
     }
 
@@ -354,53 +504,64 @@ impl<'a> client::Message for EspMessage<'a> {
     }
 }
 
-struct Queue(QueueHandle_t);
+unsafe impl Send for Newtype<esp_mqtt_event_handle_t> {}
 
-impl Drop for Queue {
-    fn drop(&mut self) {
-        unsafe {
-            vQueueDelete(self.0);
+struct EspMqttConnectionState {
+    message: Mutex<Option<Newtype<esp_mqtt_event_handle_t>>>,
+    posted: Condvar,
+    processed: Condvar,
+}
+
+#[derive(Clone)]
+pub struct EspMqttConnection(Arc<EspMqttConnectionState>);
+
+impl EspMqttConnection {
+    fn post(&self, event: esp_mqtt_event_handle_t) {
+        let mut message = self.0.message.lock();
+
+        while message.is_some() {
+            message = self.0.processed.wait(message);
+        }
+
+        *message = Some(Newtype(event));
+        self.0.posted.notify_all();
+
+        while message.is_some() {
+            message = self.0.processed.wait(message);
         }
     }
 }
 
-pub struct EspConnection(Arc<Queue>);
+unsafe impl Send for EspMqttConnection {}
 
-impl EspConnection {
-    fn post(queue: &Queue, event: esp_mqtt_event_handle_t) {
-        esp!(unsafe {
-            xQueueGenericSend(
-                queue.0,
-                &event as *const _ as *const _,
-                TickType_t::max_value(),
-                0_i32,
-            )
-        })
-        .unwrap();
-    }
+impl service::Service for EspMqttConnection {
+    type Error = EspError;
 }
 
-unsafe impl Send for EspConnection {}
-
-impl client::Connection for EspConnection {
-    type Error = EspError;
-
-    type Message<'a> = EspMessage<'a>;
+impl client::Connection for EspMqttConnection {
+    type Message<'a> = EspMqttMessage<'a>;
 
     fn next(&mut self) -> Option<Result<client::Event<Self::Message<'_>>, Self::Error>> {
-        let mut conn_data: esp_mqtt_event_handle_t = ptr::null_mut();
+        let mut message = self.0.message.lock();
 
-        esp!(unsafe {
-            xQueuePeek(
-                self.0 .0,
-                &mut conn_data as *mut _ as *mut _,
-                TickType_t::max_value(),
-            )
-        })
-        .unwrap();
+        while message.is_none() {
+            message = self.0.posted.wait(message);
+        }
 
-        let conn_data = unsafe { conn_data.as_ref() };
+        let event = unsafe { message.as_ref().unwrap().0.as_ref() };
+        if let Some(event) = event {
+            let event = EspMqttMessage::new_event(event, Some(&self.0));
 
-        conn_data.map(|event| EspMessage::new_event(event, Some(self.0.clone())))
+            if let Ok(client::Event::Received(_)) = event.as_ref() {
+                Some(event)
+            } else {
+                *message = None;
+                self.0.processed.notify_all();
+
+                Some(event)
+            }
+        } else {
+            None
+        }
     }
 }
